package jasarus.di.spring;

import jasarus.di.api.NoSuchDependencyException;
import jasarus.di.spi.DependenciesRegistry;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import java.util.Iterator;

/**
 * Dependencies registry connecting to the existing Spring container or creating new one.
 */
public class SpringDependenciesRegistry implements DependenciesRegistry {

    private final ApplicationContext applicationContext = resolveApplicationContext();

    @Override
    public <T> T lookup(Class<T> type, String qualifier) {
        try {
            return applicationContext.getBean(qualifier, type);
        } catch (BeansException e) {
            throw new NoSuchDependencyException();
        }
    }

    @Override
    public <T> T lookup(Class<T> type) {
        Iterator<T> beansOfType = applicationContext.getBeansOfType(type).values().iterator();
        if (!beansOfType.hasNext()) {
            throw new NoSuchDependencyException();
        }
        try {
            return beansOfType.next();
        } catch (BeansException e) {
            throw new NoSuchDependencyException();
        }
    }

    @Override
    public void injectInto(Object bean) {
        applicationContext.getAutowireCapableBeanFactory().autowireBean(bean);
    }

    @Override
    public void shutdown() {
        if (applicationContext instanceof ConfigurableApplicationContext) {
            ((ConfigurableApplicationContext) applicationContext).close();
        }
    }

    private ApplicationContext resolveApplicationContext() {
        ApplicationContext applicationContext = StaticApplicationContextLocator.applicationContext();
        if (applicationContext != null) {
            return applicationContext;
        }

        try {
            Class<?> bootstrapConfiguration = Class.forName("jasarus.di.spring.SpringBootstrap");
            return new AnnotationConfigApplicationContext(bootstrapConfiguration);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

}
